"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.lookupUpdates = void 0;
const datasource_1 = require("../../../../datasource");
const datasourceGitSubmodules = __importStar(require("../../../../datasource/git-submodules"));
const logger_1 = require("../../../../logger");
const manager_1 = require("../../../../manager");
const types_1 = require("../../../../types");
const clone_1 = require("../../../../util/clone");
const package_rules_1 = require("../../../../util/package-rules");
const allVersioning = __importStar(require("../../../../versioning"));
const bucket_1 = require("./bucket");
const current_1 = require("./current");
const filter_1 = require("./filter");
const rollback_1 = require("./rollback");
const update_type_1 = require("./update-type");
async function lookupUpdates(inconfig) {
    var _a, _b;
    let config = { ...inconfig };
    const { currentDigest, currentValue, datasource, depName, digestOneAndOnly, followTag, lockedVersion, packageFile, pinDigests, rollbackPrs, isVulnerabilityAlert, } = config;
    logger_1.logger.trace({ dependency: depName, currentValue }, 'lookupUpdates');
    // Use the datasource's default versioning if none is configured
    const versioning = allVersioning.get(config.versioning || datasource_1.getDefaultVersioning(datasource));
    const res = { updates: [], warnings: [] };
    // istanbul ignore if
    if (!datasource_1.isGetPkgReleasesConfig(config)) {
        res.skipReason = types_1.SkipReason.InvalidConfig;
        return res;
    }
    const isValid = currentValue && versioning.isValid(currentValue);
    if (isValid) {
        const dependency = clone_1.clone(await datasource_1.getPkgReleases(config));
        if (!dependency) {
            // If dependency lookup fails then warn and return
            const warning = {
                depName,
                message: `Failed to look up dependency ${depName}`,
            };
            logger_1.logger.debug({ dependency: depName, packageFile }, warning.message);
            // TODO: return warnings in own field
            res.warnings.push(warning);
            return res;
        }
        if (dependency.deprecationMessage) {
            logger_1.logger.debug({ dependency: depName }, 'Found deprecationMessage');
            res.deprecationMessage = dependency.deprecationMessage;
        }
        res.sourceUrl = dependency === null || dependency === void 0 ? void 0 : dependency.sourceUrl;
        if (dependency.sourceDirectory) {
            res.sourceDirectory = dependency.sourceDirectory;
        }
        res.homepage = dependency.homepage;
        res.changelogUrl = dependency.changelogUrl;
        res.dependencyUrl = dependency === null || dependency === void 0 ? void 0 : dependency.dependencyUrl;
        const { releases } = dependency;
        const latestVersion = (_a = dependency.tags) === null || _a === void 0 ? void 0 : _a.latest;
        // Filter out any results from datasource that don't comply with our versioning
        let allVersions = releases.filter((release) => versioning.isVersion(release.version));
        // istanbul ignore if
        if (allVersions.length === 0) {
            const message = `Found no results from datasource that look like a version`;
            logger_1.logger.debug({ dependency: depName, result: dependency }, message);
            if (!currentDigest) {
                return res;
            }
        }
        // Reapply package rules in case we missed something from sourceUrl
        config = package_rules_1.applyPackageRules({ ...config, sourceUrl: res.sourceUrl });
        if (followTag) {
            const taggedVersion = dependency.tags[followTag];
            if (!taggedVersion) {
                res.warnings.push({
                    depName,
                    message: `Can't find version with tag ${followTag} for ${depName}`,
                });
                return res;
            }
            allVersions = allVersions.filter((v) => v.version === taggedVersion ||
                (v.version === currentValue &&
                    versioning.isGreaterThan(taggedVersion, currentValue)));
        }
        // Check that existing constraint can be satisfied
        const allSatisfyingVersions = allVersions.filter((v) => versioning.matches(v.version, currentValue));
        if (rollbackPrs && !allSatisfyingVersions.length) {
            const rollback = rollback_1.getRollbackUpdate(config, allVersions);
            // istanbul ignore if
            if (!rollback) {
                res.warnings.push({
                    depName,
                    message: `Can't find version matching ${currentValue} for ${depName}`,
                });
                return res;
            }
            res.updates.push(rollback);
        }
        let rangeStrategy = manager_1.getRangeStrategy(config);
        // istanbul ignore next
        if (isVulnerabilityAlert &&
            rangeStrategy === 'update-lockfile' &&
            !lockedVersion) {
            rangeStrategy = 'bump';
        }
        const nonDeprecatedVersions = releases
            .filter((release) => !release.isDeprecated)
            .map((release) => release.version);
        const currentVersion = current_1.getCurrentVersion(config, versioning, rangeStrategy, latestVersion, nonDeprecatedVersions) ||
            current_1.getCurrentVersion(config, versioning, rangeStrategy, latestVersion, allVersions.map((v) => v.version));
        if (currentVersion &&
            rangeStrategy === 'pin' &&
            !versioning.isSingleVersion(currentValue)) {
            res.updates.push({
                updateType: 'pin',
                isPin: true,
                newValue: versioning.getNewValue({
                    currentValue,
                    rangeStrategy,
                    currentVersion,
                    newVersion: currentVersion,
                }),
                newMajor: versioning.getMajor(currentVersion),
            });
        }
        let filterStart = currentVersion;
        if (lockedVersion && rangeStrategy === 'update-lockfile') {
            // Look for versions greater than the current locked version that still satisfy the package.json range
            filterStart = lockedVersion;
        }
        // Filter latest, unstable, etc
        let filteredVersions = filter_1.filterVersions(config, filterStart, latestVersion, allVersions).filter((v) => 
        // Leave only compatible versions
        versioning.isCompatible(v.version, currentValue));
        if (isVulnerabilityAlert) {
            filteredVersions = filteredVersions.slice(0, 1);
        }
        const buckets = {};
        for (const release of filteredVersions) {
            const bucket = bucket_1.getBucket(config, currentVersion, release.version, versioning);
            if (buckets[bucket]) {
                buckets[bucket].push(release);
            }
            else {
                buckets[bucket] = [release];
            }
        }
        for (const [bucket, bucketReleases] of Object.entries(buckets)) {
            const sortedReleases = bucketReleases.sort((r1, r2) => versioning.sortVersions(r1.version, r2.version));
            const bucketRelease = sortedReleases.pop();
            const newVersion = bucketRelease.version;
            const update = {
                currentVersion,
                newVersion,
                newValue: null,
            };
            update.bucket = bucket;
            try {
                update.newValue = versioning.getNewValue({
                    currentValue,
                    rangeStrategy,
                    currentVersion,
                    newVersion,
                });
            }
            catch (err) /* istanbul ignore next */ {
                logger_1.logger.warn({ err, currentValue, rangeStrategy, currentVersion, newVersion }, 'getNewValue error');
                update.newValue = currentValue;
            }
            if (!update.newValue || update.newValue === currentValue) {
                if (!lockedVersion) {
                    continue; // eslint-disable-line no-continue
                }
                // istanbul ignore if
                if (rangeStrategy === 'bump') {
                    logger_1.logger.trace({ depName, currentValue, lockedVersion, newVersion }, 'Skipping bump because newValue is the same');
                    continue; // eslint-disable-line no-continue
                }
                update.currentVersion = lockedVersion;
                update.displayFrom = lockedVersion;
                update.displayTo = newVersion;
                update.isSingleVersion = true;
            }
            update.newMajor = versioning.getMajor(newVersion);
            update.newMinor = versioning.getMinor(newVersion);
            update.updateType =
                update.updateType ||
                    update_type_1.getUpdateType(config, versioning, currentVersion, newVersion);
            update.isSingleVersion =
                update.isSingleVersion || !!versioning.isSingleVersion(update.newValue);
            if (!versioning.isVersion(update.newValue)) {
                update.isRange = true;
            }
            const releaseFields = [
                'checksumUrl',
                'downloadUrl',
                'newDigest',
                'releaseTimestamp',
            ];
            releaseFields.forEach((field) => {
                if (bucketRelease[field] !== undefined) {
                    update[field] = bucketRelease[field];
                }
            });
            if (sortedReleases.length) {
                update.skippedOverVersions = sortedReleases.map((r) => r.version);
            }
            if (rangeStrategy === 'update-lockfile' &&
                currentValue === update.newValue) {
                update.isLockfileUpdate = true;
            }
            if (rangeStrategy === 'bump' &&
                versioning.matches(newVersion, currentValue)) {
                update.isBump = true;
            }
            res.updates.push(update);
        }
    }
    else if (currentValue) {
        logger_1.logger.debug(`Dependency ${depName} has unsupported value ${currentValue}`);
        if (!pinDigests && !currentDigest) {
            res.skipReason = types_1.SkipReason.InvalidValue;
        }
        else {
            delete res.skipReason;
        }
    }
    else {
        res.skipReason = types_1.SkipReason.InvalidValue;
    }
    // Record if the dep is fixed to a version
    if (lockedVersion) {
        res.fixedVersion = lockedVersion;
    }
    else if (currentValue && versioning.isSingleVersion(currentValue)) {
        res.fixedVersion = currentValue.replace(/^=+/, '');
    }
    // Add digests if necessary
    if (datasource_1.supportsDigests(config)) {
        if (currentDigest && datasource !== datasourceGitSubmodules.id) {
            if (!digestOneAndOnly || !res.updates.length) {
                // digest update
                res.updates.push({
                    updateType: 'digest',
                    newValue: currentValue,
                });
            }
        }
        else if (pinDigests) {
            // Create a pin only if one doesn't already exists
            if (!res.updates.some((update) => update.updateType === 'pin')) {
                // pin digest
                res.updates.push({
                    updateType: 'pin',
                    newValue: currentValue,
                });
            }
        }
        else if (datasource === datasourceGitSubmodules.id) {
            const dependency = clone_1.clone(await datasource_1.getPkgReleases(config));
            if ((_b = dependency === null || dependency === void 0 ? void 0 : dependency.releases[0]) === null || _b === void 0 ? void 0 : _b.version) {
                res.updates.push({
                    updateType: 'digest',
                    newValue: dependency.releases[0].version,
                });
            }
        }
        if (versioning.valueToVersion) {
            for (const update of res.updates || []) {
                update.newVersion = versioning.valueToVersion(update.newValue);
                update.currentVersion = versioning.valueToVersion(update.currentVersion);
                update.newVersion = versioning.valueToVersion(update.newVersion);
            }
        }
        // update digest for all
        for (const update of res.updates) {
            if (pinDigests || currentDigest) {
                update.newDigest =
                    update.newDigest || (await datasource_1.getDigest(config, update.newValue));
                if (update.newDigest) {
                    update.newDigestShort = update.newDigest
                        .replace('sha256:', '')
                        .substring(0, 7);
                }
                else {
                    logger_1.logger.debug({ newValue: update.newValue }, 'Could not getDigest');
                }
            }
        }
    }
    if (res.updates.length) {
        delete res.skipReason;
    }
    // Strip out any non-changed ones
    res.updates = res.updates
        .filter((update) => update.newDigest !== null)
        .filter((update) => update.newValue !== currentValue ||
        update.isLockfileUpdate ||
        (update.newDigest && !update.newDigest.startsWith(currentDigest)));
    if (res.updates.some((update) => update.updateType === 'pin')) {
        for (const update of res.updates) {
            if (update.updateType !== 'pin' &&
                update.updateType !== 'rollback' &&
                !isVulnerabilityAlert) {
                update.blockedByPin = true;
            }
        }
    }
    return res;
}
exports.lookupUpdates = lookupUpdates;
//# sourceMappingURL=index.js.map