"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.flattenUpdates = exports.applyUpdateConfig = void 0;
const config_1 = require("../../../config");
const languages_1 = require("../../../constants/languages");
const datasource_1 = require("../../../datasource");
const manager_1 = require("../../../manager");
const package_rules_1 = require("../../../util/package-rules");
const branch_name_1 = require("./branch-name");
const upper = (str) => str.charAt(0).toUpperCase() + str.substr(1);
function applyUpdateConfig(input) {
    const updateConfig = { ...input };
    delete updateConfig.packageRules;
    // TODO: Remove next line once #8075 is complete
    updateConfig.depNameShort || (updateConfig.depNameShort = updateConfig.depName);
    updateConfig.depNameSanitized = updateConfig.depName
        ? updateConfig.depName
            .replace('@types/', '')
            .replace('@', '')
            .replace(/\//g, '-')
            .replace(/\s+/g, '-')
            .replace(/-+/, '-')
            .toLowerCase()
        : undefined;
    if (updateConfig.language === languages_1.LANGUAGE_DOCKER &&
        /(^|\/)node$/.exec(updateConfig.depName) &&
        updateConfig.depName !== 'calico/node') {
        updateConfig.additionalBranchPrefix = '';
        updateConfig.depNameSanitized = 'node';
    }
    branch_name_1.generateBranchName(updateConfig);
    return updateConfig;
}
exports.applyUpdateConfig = applyUpdateConfig;
async function flattenUpdates(config, packageFiles) {
    var _a, _b;
    const updates = [];
    const updateTypes = [
        'major',
        'minor',
        'patch',
        'pin',
        'digest',
        'lockFileMaintenance',
    ];
    for (const [manager, files] of Object.entries(packageFiles)) {
        const managerConfig = config_1.getManagerConfig(config, manager);
        for (const packageFile of files) {
            const packageFileConfig = config_1.mergeChildConfig(managerConfig, packageFile);
            const packagePath = (_a = packageFile.packageFile) === null || _a === void 0 ? void 0 : _a.split('/');
            // istanbul ignore else: can never happen and would throw
            if (packagePath.length > 0) {
                packagePath.splice(-1, 1);
            }
            if (packagePath.length > 0) {
                packageFileConfig.parentDir = packagePath[packagePath.length - 1];
                packageFileConfig.packageFileDir = packagePath.join('/');
            }
            else {
                packageFileConfig.parentDir = '';
                packageFileConfig.packageFileDir = '';
            }
            for (const dep of packageFile.deps) {
                if (dep.updates.length) {
                    const depConfig = config_1.mergeChildConfig(packageFileConfig, dep);
                    delete depConfig.deps;
                    for (const update of dep.updates) {
                        let updateConfig = config_1.mergeChildConfig(depConfig, update);
                        delete updateConfig.updates;
                        updateConfig.newVersion =
                            updateConfig.newVersion || updateConfig.newValue;
                        if (updateConfig.updateType) {
                            updateConfig[`is${upper(updateConfig.updateType)}`] = true;
                        }
                        if (updateConfig.updateTypes) {
                            updateConfig.updateTypes.forEach((updateType) => {
                                updateConfig[`is${upper(updateType)}`] = true;
                            });
                        }
                        // apply config from datasource
                        const datasourceConfig = await datasource_1.getDefaultConfig(depConfig.datasource);
                        updateConfig = config_1.mergeChildConfig(updateConfig, datasourceConfig);
                        updateConfig = package_rules_1.applyPackageRules(updateConfig);
                        // apply major/minor/patch/pin/digest
                        updateConfig = config_1.mergeChildConfig(updateConfig, updateConfig[updateConfig.updateType]);
                        for (const updateType of updateTypes) {
                            delete updateConfig[updateType];
                        }
                        // Apply again in case any were added by the updateType config
                        updateConfig = package_rules_1.applyPackageRules(updateConfig);
                        updateConfig = applyUpdateConfig(updateConfig);
                        updateConfig.baseDeps = packageFile.deps;
                        updates.push(updateConfig);
                    }
                }
            }
            if (manager_1.get(manager, 'supportsLockFileMaintenance') &&
                packageFileConfig.lockFileMaintenance.enabled) {
                // Apply lockFileMaintenance config before packageRules
                let lockFileConfig = config_1.mergeChildConfig(packageFileConfig, packageFileConfig.lockFileMaintenance);
                lockFileConfig.updateType = 'lockFileMaintenance';
                lockFileConfig = package_rules_1.applyPackageRules(lockFileConfig);
                // Apply lockFileMaintenance and packageRules again
                lockFileConfig = config_1.mergeChildConfig(lockFileConfig, lockFileConfig.lockFileMaintenance);
                lockFileConfig = package_rules_1.applyPackageRules(lockFileConfig);
                // Remove unnecessary objects
                for (const updateType of updateTypes) {
                    delete lockFileConfig[updateType];
                }
                delete lockFileConfig.packageRules;
                delete lockFileConfig.deps;
                branch_name_1.generateBranchName(lockFileConfig);
                updates.push(lockFileConfig);
            }
            if (manager_1.get(manager, 'updateLockedDependency')) {
                for (const lockFile of packageFileConfig.lockFiles || []) {
                    const remediations = (_b = config.remediations) === null || _b === void 0 ? void 0 : _b[lockFile];
                    if (remediations) {
                        for (const remediation of remediations) {
                            let updateConfig = config_1.mergeChildConfig(packageFileConfig, remediation);
                            updateConfig = config_1.mergeChildConfig(updateConfig, config.vulnerabilityAlerts);
                            updateConfig.isVulnerabilityAlert = true;
                            updateConfig.isRemediation = true;
                            updateConfig.lockFile = lockFile;
                            updateConfig.currentValue = updateConfig.currentVersion;
                            updateConfig.newValue = updateConfig.newVersion;
                            updateConfig = applyUpdateConfig(updateConfig);
                            updates.push(updateConfig);
                        }
                    }
                }
            }
        }
    }
    return updates
        .filter((update) => update.enabled)
        .map(({ vulnerabilityAlerts, ...update }) => update)
        .map((update) => config_1.filterConfig(update, 'branch'));
}
exports.flattenUpdates = flattenUpdates;
//# sourceMappingURL=flatten.js.map